@@ -275,6 +275,10 @@ _detectDE_XDG_CURRENT() {
             DE=deepin
             return
             ;;
+            X-Cinnamon)
+            DE=cinnamon
+            return
+            ;;
         esac
     done
     return 1
@@ -317,6 +321,9 @@ _detectDE_SESSION() {
         Deepin)
             DE=deepin
             ;;
+        cinnamon)
+            DE=cinnamon
+            ;;
         *)
             return 1
             ;;
@@ -799,8 +806,7 @@ check_system() {
             "$(code_inline "${DE}")"
     fi
     write_order_list "$(_ 'XDG SESSION TYPE:')"
-    write_quote_str "XDG SESSION TYPE='${XDG_SESSION_TYPE}'"
-
+    write_quote_str "XDG_SESSION_TYPE='${XDG_SESSION_TYPE}'"
     write_order_list "$(_ 'Bash Version:')"
     write_quote_str "BASH_VERSION='${BASH_VERSION}'"
 }
@@ -1210,7 +1216,7 @@ find_qt_modules() {
 
 check_qt() {
     write_title 2 "Qt:"
-    _check_toolkit_env qt4 QT4_IM_MODULE QT_IM_MODULEb/data/fcitx5-diagnose.sh
+    _check_toolkit_env qt4 QT4_IM_MODULE QT_IM_MODULE
     _check_toolkit_env qt5 QT_IM_MODULE
     find_qt_modules
     qt4_module_found=''
@@ -1242,7 +1248,7 @@ check_qt() {
         elif [[ ${file} =~ /fcitx5/qt ]]; then
             write_eval "$(_ 'Found ${1} ${2} module: ${3}.')" \
                         fcitx5 ${qt_version} "$(code_inline "${file}")"
-        elseb/data/fcitx5-diagnose.sh
+        else
             write_eval "$(_ 'Found unknown ${1} qt module: ${2}.')" \
                        fcitx \
                        "$(code_inline "${file}")"
